// VGA Microcontroller
// for Philips VGA Module 

#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/io.h>

// Hsync, wall and Q1-Q4 values Prescale = 8
#define h_Line 636 // Cycles for one line: Cycle 20 MHz divided by 31.477 KHz - 800 lines

#define h_wall_right_start 60 

#define h_sync_end 3 // 3

#define h_line_length 636 // 76

#define h_cycle 27
#define h_frontporch 25


#define v_line_count 523
#define v_visible_end 480
#define v_sync_start 491
#define v_sync_end 493

//Makro for video handle
//HSYNC
#define hsync_on  PORTD = 0x80;   // D7 (HS)
#define hsync_off  PORTD = 0x00;   // all off (HS)


//VSYNC
#define vsync_on  PORTB = 0x03;   // B0 & B1 (VS)
#define vsync_off  PORTB = 0x00;   // all off (VS)
 
//horizontal Outputs
#define h_0 PORTD = 0x01; // D0 (Whl)
#define h_1 PORTD = 0x00; // all off
#define h_2 PORTD = 0x20; // D5 (Q1)
#define h_3 PORTD = 0x10; // D4 (Q2)
#define h_4 PORTD = 0x30; // D4 and D5 (Q1 and Q2)
#define h_5 PORTD = 0x08; // D3 (Q3)
#define h_6 PORTD = 0x28; // D3 and D5 (Q3 and Q1)
#define h_7 PORTD = 0x18; // D3 and D5 (Q3 and Q2)
#define h_8 PORTD = 0x38; // D3 and D4 and D5 (Q3 and Q2 and Q1)
#define h_9 PORTD = 0x04; // D2 (Q4)
#define h_10 PORTD = 0x24; // D2 and D5 (Q4 and Q1)
#define h_11 PORTD = 0x14; // D2 and D4 (Q4 and Q2)
#define h_12 PORTD = 0x34; // D2 and D4 and D5 (Q4 and Q1 and Q2)
#define h_13 PORTD = 0x0C; // D2 and D3 (Q4 and Q3)
#define h_14 PORTD = 0x2C; // D2 and D3 and D5 (Q4 and Q3 and Q1)
#define h_15 PORTD = 0x1C; // D2 and D3 and D4 (Q4 and Q3 and Q2)
#define h_16 PORTD = 0x3C; // D2 and D3 and D4 and D5 (Q4 and Q3 and Q2 and Q1)
#define h_17 PORTD = 0x3E; // D1 and D2 and D3 and D4 and D5 (Whr and Q4 and Q3 and Q2 and Q1)
#define h_18 PORTD = 0x00; // all off
#define h_off PORTD = 0x00; // all off


//vertical Outputs
#define v_0 PORTC = 0x10; // C4 (Wvt)
#define v_1 PORTC = 0x00; // all off
#define v_2 PORTC = 0x01; // C0 (Q5)
#define v_3 PORTC = 0x02; // C1 (Q6)
#define v_4 PORTC = 0x03; // C2 and C0 (Q5 and Q6)
#define v_5 PORTC = 0x04; // C2 (Q7)
#define v_6 PORTC = 0x05; // C2 and C0 (Q7 and Q5)
#define v_7 PORTC = 0x06; // C2 and C1 (Q7 and Q6)
#define v_8 PORTC = 0x07; // C2 and C1 and C0 (Q7 and Q6 and Q5)
#define v_9 PORTC = 0x08; // C3 (Q8)
#define v_10 PORTC = 0x09; // C3 and C0 (Q8 and Q5)
#define v_11 PORTC = 0x0A; // C3 and C1 (Q8 and Q6)
#define v_12 PORTC = 0x0B; // C3 and C1 and C2 (Q8 and Q5 and Q6)
#define v_13 PORTC = 0x0C; // C3 and C2 (Q8 and Q7)
#define v_14 PORTC = 0x0D; // C3 and C2 and C1 (Q8 and Q7 and Q5)
#define v_15 PORTC = 0x0E; // C3 and C2 and C2 (Q8 and Q7 and Q6)
#define v_16 PORTC = 0x0F; // C3 and C2 and C1 and C0 (Q8 and Q7 and Q6 and Q5)
#define v_17 PORTC = 0x2F; // C5 and C3 and C2 and C1 and C0 (Wvb and Q8 and Q7 and Q6 and Q5)
#define v_18 PORTC = 0x00; // all off
#define v_off = 0x00; // all off

#define NOP asm("nop")
#define NOP10 asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
#define NOP8 asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");  asm("nop"); asm("nop"); 
#define NOP6 asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");   

//Global variables
volatile unsigned int linecount;// current line

static void avr_init(void)
{
	
//Set power mode
  set_sleep_mode(SLEEP_MODE_IDLE); 	
//init Ports

// Port B: B0
  DDRB = 0x01;
  PORTB = 0x00;
  
// Port C0..C5 = 0x3F  
  
  DDRC = 0x3F;
  PORTC = 0x00;

// Port D: all but D6 = 0xBF
  DDRD = 0xBF;
  PORTD = 0x00;
  
  return;
}
    

// All VGA Modules signals are created here



ISR(TIMER1_COMPA_vect)
{


	h_0;  
	NOP10;
	h_2;

	NOP10;
	NOP10;
	NOP8;
	
	h_3;  
	NOP10;
	NOP10;
	NOP8;
	h_4;
	NOP10;
	NOP10;
	NOP8;
	h_5;  
	NOP10;
	NOP8;
	NOP10;
	h_6;
	NOP10;
	NOP10;
	NOP8;
	h_7;  
	NOP10;
	NOP10;
	NOP8;
	h_8;
	NOP10;
	NOP10;
	NOP8;
    h_9;  
	NOP10;
	NOP10;
	NOP8;
	h_10;
	NOP10;
	NOP10;
	NOP8;
	h_11;  
	NOP10;
	NOP10;
	NOP6;
	h_12;
	NOP10;
	NOP10;
	NOP8;
	h_13;  
	NOP10;
	NOP10;
	NOP6;
	h_14;
	NOP10;
	NOP10;
	NOP8;
	h_15;  
	NOP10;
	NOP10;
	NOP6;
	h_16;
	NOP10;
	NOP8;
	h_17;
	NOP10;
	h_18;


// horizontal Front Porch : 16 pixels = 12 cycles
	NOP10;
		
	NOP;
	NOP;
	NOP;
	

// horizontal sync : 96 pixels = 76 cycles
	
	hsync_on;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //10
	
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //20
	
	
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //30
	
	
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //40
	
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //50
	
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //60
	
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP; //70
	

	NOP;   
	NOP;
	NOP;   
	NOP;

	hsync_off;	
	

// Back Porch - use for vertical sync	
	
	
	if (++linecount == v_line_count)
	{
		linecount = 0;   //Reset linecount on maximum
	} 
	

	switch (linecount)
	{
		case 0: 
			vsync_on;
			break;
		case 3: 
			vsync_off;
			break;

		default: 
			break;
		
	}




	



	



	
// Enable Interrupts	
//	sei();
		

}







int main(void)
{
// Initialize ports
    avr_init();
	
	
	
// Compare Register
	OCR1A = 636; // Complete Line Length

//start timer with prescaler = 1
	TCCR1B = (1<<CS10) | (1<<WGM12);

//enable output compare interrupt for OCR1A
	TIMSK1 = (1<<OCIE1A);
	
// enable Interrupts
	sei();

    while(1)
	{
// loop forever timer does the job
    }

	return(0);
}





	
    